#include <cctype>
#include <cstdio>
#include <cstring>
#include <list>
#include <vector>

using namespace std;

int k, n;

enum {
	VERBATIM,
	UNIQUE,
	ANY
};

#define EPS	('a' - 1)

int sw_symb_to_int(char sw_symb)
{
	return sw_symb - EPS;
}

class symbol
{
private:
	char a, b;
public:
	symbol() { }
	symbol(char c) : a('\0'), b(c) { }
	symbol(char c1, char c2) : a(c1), b(c2) { }
	bool is_double() const { return a != '\0'; }
	char letter() const { return b; };
	char letter2() const { return a; }
};

class codes_set
{
private:
	const int k;
	const int n;
	
	struct index
	{
		int pos;
		char sw_symb;
		int state;
		index(int pos, char sw_symb, int state)
		: pos(pos), sw_symb(sw_symb), state(state) { }
	};
	
	int get_idx(int pos, char sw_symb, int state)
	{
		return (k + 1) * (3 * pos + state) + sw_symb_to_int(sw_symb);
	}
	
	int get_idx(index idx)
	{
		return get_idx(idx.pos, idx.sw_symb, idx.state);
	}
	
	struct info {
		int len;
		symbol s;
		int prev;
		list<index>::iterator ptr;
		info() : len(INT_MAX) { }
		info(int len, list<index>::iterator ptr)
		: len(len), prev(-1), ptr(ptr) { }
	};
	
	vector<info> v;
	int cur_len;
	list<index> _li[3];
	list<index>* li[3];
	
	void swap_lists()
	{
		list<index>* tmp = li[0];
		li[0] = li[1];
		li[1] = li[2];
		li[2] = tmp;
		cur_len++;
	}
public:
 	class code;
	
	class code_iterator
	{
	private:
		friend class code;
		const codes_set &S;
		int vidx;
		code_iterator(const codes_set& S, int vidx)
		: S(S), vidx(vidx) { }
	public:
		bool first() const { return S.v[vidx].prev < 0; }
		symbol symb() const { return S.v[vidx].s; }
		code_iterator& operator--()
		{
			vidx = S.v[vidx].prev;
			return *this;
		}
	};

	class code
	{
	private:
		friend class codes_set;
		const codes_set& S;
		const index idx;
		const int vidx;
		code(codes_set& S, index idx, int vidx)
		: S(S), idx(idx), vidx(vidx) { }
	public:
		int pos() const { return idx.pos; }
		char sw_symb() const { return idx.sw_symb; }
		int state() const { return idx.state; }
		int len() const { return S.v[vidx].len; }
		code_iterator iterator() const
		{ return code_iterator(S, vidx); }
	};
	
	codes_set(int k, int n)
	: k(k), n(n)
	{
		v = vector<info>((k+1) * (n+1) * 3, info());
		cur_len = 0;
		for (int i = 0; i < 3; i++)
			li[i] = &_li[i];
		index idx(0, EPS, ANY);
		_li[0].push_back(idx);
		v[get_idx(idx)] = info(0, --_li[0].end());
	}
	
	code pop_shortest()
	{
		while (li[0]->empty())
			swap_lists();
		index idx = li[0]->front();
		li[0]->pop_front();
		return code(*this, idx, get_idx(idx));
	}
	
	void update(int pos, char sw_symb, int state,
		    int len, code &prev, symbol s)
	{
		index idx(pos, sw_symb, state);
		int vidx = get_idx(idx);
		info &i = v[vidx];
		if (len < i.len) {
			list<index>& l = *li[len-cur_len];
			if (i.len == INT_MAX) {
				l.push_back(idx);
				i.ptr = --l.end();
			} else
				l.splice(l.end(), *li[i.len-cur_len], i.ptr);
			i.len = len;
			i.s = s;
			i.prev = prev.vidx;
		}
	}
};

void print_solution(int len, codes_set::code_iterator ci)
{
	char sol[len + 1];
	char *solptr = sol + len;
	*solptr = '\0';
	for (; !ci.first(); --ci) {
		symbol s = ci.symb();
		*--solptr = s.letter();
		if (s.is_double())
			*--solptr = s.letter2();
	}
	printf("%s\n", sol);
}

void solve(int k, int n, char *w)
{
	codes_set S(k, n);
	while (true) {
		codes_set::code c = S.pop_shortest();
		if (c.pos() == n) {
			print_solution(c.len(), c.iterator());
			break;
		}

		char l = w[c.pos()];
		if (c.state() == VERBATIM) {
			if (islower(l) && l != c.sw_symb())
				S.update(c.pos()+1, c.sw_symb(), VERBATIM,
					 c.len()+1, c, symbol(l));
			S.update(c.pos(), c.sw_symb(), ANY,
				 c.len()+1, c, symbol(c.sw_symb()));
				
		} else {
			if (isupper(l) && tolower(l) != c.sw_symb() &&
			    !(c.state() == UNIQUE && l == w[c.pos()-1]))
				S.update(c.pos()+1, c.sw_symb(), UNIQUE,
					 c.len()+1, c, symbol(tolower(l)));
			if (c.sw_symb() != EPS)
				S.update(c.pos(), c.sw_symb(), VERBATIM,
					 c.len()+1, c, symbol(c.sw_symb()));
			for (char s = 'a'; s < 'a' + k; s++)
				if (!(c.state() == UNIQUE &&
				      s == tolower(w[c.pos()-1])))
					S.update(c.pos(), s, ANY,
						 c.len()+2, c, symbol(s, s));
		}
	}
}

int main()
{
	stdin = fopen("encoding.in", "r");
	stdout = fopen("encoding.out", "w");
	scanf("%d\n", &k);
	char w[100001];
	scanf("%s", w);
	n = strlen(w);
	solve(k, n, w);
	return 0;
}
