#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "common.h"
#include "parse.h"
#include "execute.h"

int debug;

int parse_input_and_execute(char *where) {
  ulng input,result=0L;
  int cmp_result=0;
  long long int parsed;
  int base;
  char *ptr;

#define parse_ulng(to) while (isspace(*where)) where++;                    \
                       base=10;                                            \
                       if (*where=='#') base=16,where++;                   \
                       parsed=strtoll(where,&ptr,base);                    \
                       if (parsed<0 || parsed>0xFFFFFFFF || ptr==where) {  \
                         fprintf(stderr,"Cannot parse input %s!\n",where); \
                         return 0;                                         \
		       }                                                   \
                       where=ptr;                                          \
                       to=(ulng)parsed;
                         

  parse_ulng(input);
  while (isspace(*where)) where++;
  if (*where=='=') {
    cmp_result=1;
    where++;
    parse_ulng(result);
  }

  if (debug) {
    printf("Running program with input %lu",input);
    if (cmp_result) printf(" comparing to %lu",result);
    printf(".\n");
  }
  return execute(input,cmp_result,result,debug);
}

int main(int argc, char *argv[]) {
  int i=1;
  FILE *prog_f;
  FILE *tmp_f;
  char *ins_filename=NULL;

  //switches
  while (argv[i] && *argv[i]=='-') switch (argv[i][1]) {
  case 'g':debug=1;
           i++;
           break;
  case 'r':if (!argv[i][2] && !argv[i+1]) {fprintf(stderr,ERR_USAGE);return 1;}
           max_ins_executed=atoi((argv[i][2])? argv[i]+2 : argv[++i] );
	   if (max_ins_executed<1) {fprintf(stderr,"Value for -r is must be bigger than 0!\n");
                                    return 1;}
           i++;
           break;
  case 'i':if (!argv[i][2] && !argv[i+1]) {fprintf(stderr,ERR_USAGE);return 1;}
           ins_filename=(argv[i][2]) ? argv[i]+2 : argv[++i];
	   i++;
           break;
  default:fprintf(stderr,ERR_USAGE);return 1;
  }
  if (debug) printf("Debug is on.\n");

  //open the program file
  if (!argv[i]) {fprintf(stderr,ERR_USAGE);return 1;}
  if (!(prog_f=fopen(argv[i],"r"))) {fprintf(stderr,"Cannot open file %s!\n",argv[i]);return 1;}
  i++;

  //parse it
  if (!parse(prog_f)) return 1; //it fprintfs the error itself
  fclose(prog_f);
  printf("Program with %d instructions successfully parsed.\n",prog_len);

 //and executions
  for(;argv[i];i++) {
    tmp_f=NULL;
    if (!strcmp(argv[i],"-")) tmp_f=stdin;
    else if (strchr(argv[i],'.')) if (!(tmp_f=fopen(argv[i],"r"))) 
      {fprintf(stderr,"Cannot open file %s!\n",argv[i]);return 1;}

    //value at the command line
    if (!tmp_f && !parse_input_and_execute(argv[i])) return 1;

    //or in the file
    if (tmp_f) {
      char line[MAX_INPUT_LINE_LEN];

      while (fgets(line,MAX_INPUT_LINE_LEN,tmp_f)) {	
	//chomp '\n'
	int len=strlen(line);	
	if (len && line[len-1]=='\n') line[len-1]='\0';
	if (!parse_input_and_execute(line)) {fclose(tmp_f);return 1;}
      }

      fclose(tmp_f);
    }
  }

  //already parsed - should write the nr of ins?
  if (ins_filename) {
    if (!(tmp_f=fopen(ins_filename,"w"))) {fprintf(stderr,"Cannot open file %s!\n",ins_filename);return 1;}
    fprintf(tmp_f,"%d\n",prog_len+1);
    fclose(tmp_f);
  }

  //we are successfully done!
  return 0;
}






