#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "common.h"
#include "parse.h"

struct label {
  char name[MAX_LABEL_LEN];
  ulng offset;
};
struct label labels[MAX_INS_NR];
int labels_len;

struct pending_label {
  char name[MAX_LABEL_LEN];
  struct value *where;
};
struct pending_label pendings[MAX_INS_NR*4]; //max 4 na instrukci 2x if, 2x prirazeni
int pendings_len;

FILE *prog_f;


#define PARSE_ERROR  0
#define PARSE_OK     1
#define PARSE_END    2
#define MAX_TOKEN_LEN MAX_LABEL_LEN

//read a token - consist os characters
//with same class (letters, digits,specials (<<<, +, ...))
//returns only PARSE_OK and PARSE_ERROR
int parse_token(char *str,int str_len) {
  int c;
  int len=0;

#define parse_add_char(c)  { \
  if (len+1>=str_len)        \
    {str[len]='\0';fprintf(stderr,"Too long token %s!",str);return PARSE_ERROR;} \
  else str[len++]=c;       }

  while (c=tolower(fgetc(prog_f)),isspace(c) || c=='\'') {
    if (c=='\'') while (c=fgetc(prog_f),c!='\n' && c!=EOF) {} //comment - skip line
  }

  if (c==EOF) {fprintf(stderr,"Unexpected end of file!\n");return PARSE_ERROR;}

  str[len++]=c;
  if (isalpha(c)) while (c=tolower(fgetc(prog_f)),isalnum(c)) parse_add_char(c)
  else if (isdigit(c)) while (c=fgetc(prog_f),isalnum(c)) parse_add_char(c)
  else if (c=='#') while (c=fgetc(prog_f),isxdigit(c)) parse_add_char(c)
  else while (c=fgetc(prog_f),!isalnum(c) && c!=EOF && !isspace(c) && c!='\'' && c!='#') parse_add_char(c)

  str[len]='\0';  
  ungetc(c,prog_f);
  return PARSE_OK;
}

//tries to fill struct value
//returns only PARSE_OK and PARSE_ERROR
int parse_value(struct value *value)
{
  char tkn[MAX_TOKEN_LEN];
  int c;

  if (!parse_token(tkn,MAX_TOKEN_LEN)) return PARSE_ERROR;
  if (isdigit(*tkn) || *tkn=='#') { //it will be a constant value
    long long int parsed;
    char *ptr;

    if (*tkn=='#') parsed=strtoll(tkn+1,&ptr,16);
    else parsed=strtoll(tkn,&ptr,10);
    if (parsed < 0 || parsed > 0xFFFFFFFF || *ptr)
      {fprintf(stderr,"Bad constant %s!\n",tkn);return PARSE_ERROR;}
				  
    value->type=VALUE_TYPE_NUM;
    value->value=(ulng) parsed;
  } else {                         //still, register or index
    while (c=fgetc(prog_f),isspace(c)) {}
    if (c=='[') {  //index
      if (!parse_value(value)) return PARSE_ERROR;  //parse the inner part
      
      while (c=fgetc(prog_f),isspace(c)) {}
      if (c!=']') {fprintf(stderr,"Error near %s[]!\n",tkn);return PARSE_ERROR;}

      if (value->type & VALUE_TYPE_INDEX)
	{fprintf(stderr,"Nested indexing near %s[] not allowed!\n",tkn);return PARSE_ERROR;}
      value->type|=VALUE_TYPE_INDEX;

      //set pending label
      strcpy(pendings[pendings_len].name,tkn);
      pendings[pendings_len++].where=value;
    } else {       //register
      ungetc(c,prog_f);

      if (strlen(tkn)!=1 || !isalpha(*tkn)) 
	{fprintf(stderr,"Bad register name %s!\n",tkn);return PARSE_ERROR;}

      value->type=VALUE_TYPE_REG;
      value->value=(ulng) *tkn-'a';
    }
  }

  return PARSE_OK;
}

int parse_command(struct ins *ins,ulng line) {
  char tkn[MAX_TOKEN_LEN];
  int c,j;

#define parse_next_nonspace(c) while (c=fgetc(prog_f),isspace(c) || c=='\'') {if (c=='\'') while (c=fgetc(prog_f),c!=EOF && c!='\n') {}}

  //end of input?
  parse_next_nonspace(c);
  if (c==EOF) return PARSE_END;
  ungetc(c,prog_f);

  if (!parse_token(tkn,MAX_TOKEN_LEN)) return PARSE_ERROR;

  //at first label
  parse_next_nonspace(c);

  if (c==':') {
    //duplicated?
    for (j=0;j<labels_len;j++) if (!strcmp(labels[j].name,tkn))
      {fprintf(stderr,"Duplicated label %s!\n",tkn);return PARSE_ERROR;}

    strcpy(labels[labels_len].name,tkn);
    labels[labels_len++].offset=line;

    //next shot, sir
    if (!parse_token(tkn,MAX_TOKEN_LEN)) return PARSE_ERROR;
  } else ungetc(c,prog_f);

  //secondly, if
  if (!strcmp(tkn,"if")) {
    //first value
    if (!parse_value(&ins->if_first)) return PARSE_ERROR;

    //now operator
    if (!parse_token(tkn,MAX_TOKEN_LEN)) return PARSE_ERROR;
    for (j=1;j<IF_TYPE_NUM;j++) if (!strcmp(if_type_desc[j],tkn)) break;
    if (j<IF_TYPE_NUM) ins->if_type=j;
    else {fprintf(stderr,"Unknown relation %s!\n",tkn);return PARSE_ERROR;}

    //and second value
    if (!parse_value(&ins->if_second)) return PARSE_ERROR;
    
    //next shot, sir
    if (!parse_token(tkn,MAX_TOKEN_LEN)) return PARSE_ERROR;
  }

  //now the command
  if (!strcmp(tkn,"halt")) {
    ins->type=INS_TYPE_HALT;
  } else
  if (!strcmp(tkn,"data")) {
    ins->type=INS_TYPE_DATA;

    if (!parse_value(&ins->first)) return PARSE_ERROR;
    if (ins->first.type != VALUE_TYPE_NUM)
      {fprintf(stderr,"Data can be used only with constant expressions!\n");return PARSE_ERROR;}
  } else
  if (!strcmp(tkn,"jump")) {
    ins->type=INS_TYPE_JMP;

    if (!parse_token(tkn,MAX_TOKEN_LEN)) return PARSE_ERROR;
    //set pending label
    strcpy(pendings[pendings_len].name,tkn);
    pendings[pendings_len++].where=&ins->first;    
  } else {  //assignment
    ins->type=INS_TYPE_ASSIGN;

    //where to assign
    if (strlen(tkn)!=1 || !isalpha(*tkn)) {
      parse_next_nonspace(c);
      if (c=='=') fprintf(stderr,"Bad register %s!\n",tkn);
      else fprintf(stderr,"Bad command %s!\n",tkn);
      return PARSE_ERROR;
    }
    ins->reg=*tkn-'a';

    //now '='
    if (!parse_token(tkn,MAX_TOKEN_LEN)) return PARSE_ERROR;
    if (strlen(tkn)!=1 || *tkn!='=') 
      {fprintf(stderr,"Bad command %s, expected '='!\n",tkn);return PARSE_ERROR;}

    //first value
    if (!parse_value(&ins->first)) return PARSE_ERROR;

    //now operator
    //well, it can happen it won't be here - read first non-space char
    //and decide
    parse_next_nonspace(c);
    ungetc(c,prog_f);
    for (j=0;j<INS_OP_NUM;j++) if (*ins_op_desc[j]==c) break;
    if (j>=INS_OP_NUM) {//no known operator begins with character 'c'
      ins->op=INS_OP_ASSIGN;
      return PARSE_OK;  
    }

    if (!parse_token(tkn,MAX_TOKEN_LEN)) return PARSE_ERROR;
    for (j=0;j<INS_OP_NUM;j++) if (!strcmp(ins_op_desc[j],tkn)) break;
    if (j<INS_OP_NUM) ins->op=j;
    else {fprintf(stderr,"Unknown operator %s!\n",tkn);return PARSE_ERROR;}

    //and second value
    if (!parse_value(&ins->second)) return PARSE_ERROR;
  }
  return PARSE_OK;
}

//just simple wrapper for parse_command
int parse_input(void) {
  int ret;
  ulng line=0;

  for(;;) {
    if (line >= MAX_INS_NR) {fprintf(stderr,"Too big program!\n");return 0;}

    ret=parse_command(prog+prog_len,line);
    if (ret==PARSE_ERROR) return 0;
    if (ret==PARSE_END) break;
    prog_len++;
    line++;
  }

  return 1;
}

int parse(FILE *f) {
  int i,j;

  prog_f=f;

  //first pass
  if (!parse_input()) return 0;

  //and now pending labels
  for (i=0;i<pendings_len;i++) {
    for (j=0;j<labels_len;j++) if (!strcmp(labels[j].name,pendings[i].name)) {
      pendings[i].where->offset=labels[j].offset;
      break;
    }
    if (j>=labels_len) {fprintf(stderr,"Undefined label %s!\n",pendings[i].name);return 0;}
  }

  return 1;
}
