
#include <stdio.h>
#include <stdlib.h>

#define MAXN (100 * 1024)


int N, Q;

struct edge {
	int to;
	int cost;
	struct edge *next;
} edges[MAXN * 2];

struct edge * edgePts[MAXN];
int numEdges = 0;

void addEdge(int from, int to, int cost) {
	edges[numEdges].to = to;
	edges[numEdges].cost = cost;
	edges[numEdges].next = edgePts[from];
	edgePts[from] = edges + numEdges;
	numEdges ++;
}

void loadGraph(FILE * f) {

	int i, nm1;
	
	fscanf(f, "%d%d", &N, &Q);

	nm1 = N - 1;
	
	for (i = 0; i < nm1; i++) {
		int a,b,c;
		fscanf(f, "%d%d%d", &a, &b, &c);
		a--;
		b--;
		addEdge(a,b,c);
		addEdge(b,a,c);
	}
}

#define MAXSEQPOWERTWO	(1 << (10 + 8 /* log_2 256 000 */))

int seq[2 * MAXN + 10];
int mins[2 * MAXSEQPOWERTWO + 1];
int minTopShift;
int seqLen = 0;
int vertex2Pos[MAXN];
int level[MAXN];
int lenToRoot[MAXN];

int xmin(int v, int w) {

	if (v < 0) {
		return w;
	}

	if (w < 0) {
		fprintf(stderr,"w problem");
		exit(1);   /* never returns */
	}
	
	if (level[v] > level[w]) {
		return w;
	} else if (level[v] < level[w]) {
		return v;
	} else if (v == w) {
		return v;
	}

	fprintf(stderr,"Oops! 2 neighboring intevals w/o unique min!");
	exit(1);   /* never returns */
	return v;
}



int dfs(int v, int xlen, int xlev) {
	struct edge *cur;

	if (level[v] >= 0) {
		return 0;
	}

	vertex2Pos[v] = seqLen;
	seq[seqLen ++] = v;
	level[v] = xlev;
	lenToRoot[v] = xlen;

	for (cur = edgePts[v]; cur; cur = cur->next) {
		if(dfs(cur->to, cur->cost + xlen, xlev + 1)) {
			seq[seqLen ++] = v;
		}
	}
	return 1;
}


void precalc() {
	int i;

	for (i = 0; i < N; i++) {
		level[i] = -1;
	}
	dfs(0,0,0);

	minTopShift = 1;
	while (minTopShift < seqLen) {
		minTopShift *= 2;
	}

	if (minTopShift > MAXSEQPOWERTWO) {
		fprintf(stderr,"Oops! 2 neighboring intevals w/o unique min!");
		exit(1);   /* never returns */
	}

	for (i = 0; i < seqLen; i++) {
		mins[i + minTopShift] = seq[i];
	}
	for (; i < minTopShift; i++) {
		mins[i + minTopShift] = -1;
	}

	for (i = minTopShift - 1; i > 0 ; i--) {
		int a = mins[2 * i];
		int b = mins[2 * i + 1];
		if (b < 0) {
			mins[i] = a; 
		} else {
			mins[i] = xmin(a,b); 
		}
	}
}


int gcp(int from, int to) {

	int l, r, s;
	int theGcp;
	
	theGcp = -1;
	from = vertex2Pos[from];
	to = vertex2Pos[to];

	if (from > to) {
		int p = from; from = to; to = p;
	}

	l = r = 1;
	s = minTopShift / 2;
   	
	while ((from & s) == (to & s)) {
		l *= 2;
		l += (from & s) ? 1 : 0;
		r = l;
		s /= 2;
	}

	l *= 2;
	r = r * 2 + 1;
	s /= 2;

	while (s) {

		l *= 2;
		if ((from & s) == 0) {
			theGcp = xmin(theGcp,mins[l + 1]);
		} else {
			l ++;
		}

		r *= 2;
		if ((to & s) != 0) {
			theGcp = xmin(theGcp,mins[r]);
			r++;
		}
		
		s /= 2;
	}

	theGcp = xmin(theGcp,mins[l]);
	theGcp = xmin(theGcp,mins[r]);

	if (theGcp < 0) {
		fprintf(stderr,"gcp problem: %d:%d\n", from, to);
		exit(1);   /* never returns */
	}

	return theGcp;
}


int query(int from, int to) {
	return lenToRoot[from] + lenToRoot[to] - 2* lenToRoot[gcp(from,to)];
}


int main() {
	
	int i;

	FILE *in = fopen("railway.in","rt");
	FILE *out = fopen("railway.out","wt");

	if (!in || !out) {
		fprintf(stderr,"Something fishy.\n");
		exit(1);   /* never returns */
	}


	loadGraph(in);
	precalc();

	for (i = 0; i < Q; i++) {
		int a,b;
		fscanf(in, "%d%d", &a,&b);
		a--; b--;
		fprintf(out, "%d\n", query(a,b));
	}
	
	return 0;
}
