#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

// FIXME: The number of corridors

static void __attribute__((noreturn)) bad(char *msg, ...)
{
  va_list args;
  va_start(args, msg);
  printf("WRONG ");
  vprintf(msg, args);
  printf("\n");
  va_end(args);
  exit(1);
}

#define MAXN 250000
#define MAXM 1000000
struct edge {
  int next;
  int to;
  int len;
  int delta;
} edges[2*MAXM+1];
int first[MAXN+1], dist[MAXN+1], done[MAXN+1], h[2*MAXM+1], path[MAXN+1], from[MAXN+1];
int X, Y, N, Tmax;
int H;
#define INF 1000111111

static void readin(FILE *f)
{
  fscanf(f, "%d%d%d%d", &X, &Y, &N, &Tmax);
  int i = 1;
  int dmax = 0;
  for (;;)
    {
      int x, y, l, d;
      fscanf(f, "%d", &x);
      if (x < 0)
	break;
      fscanf(f, "%d%d%d", &y, &l, &d);
      if (x <= 0 || x > N || y <= 0 || y > N || l <= 0 || l > 1000000 || d < 0 || d > 1)
	bad("Incorrect input (%d,%d,%d,%d)", x, y, l, d);
      if (d > dmax)
	dmax = d;
      edges[i].to = y;
      edges[i].len = l;
      edges[i].delta = d;
      edges[i].next = first[x];
      first[x] = i++;
      edges[i].to = x;
      edges[i].len = l;
      edges[i].delta = d;
      edges[i].next = first[y];
      first[y] = i++;
      if (i > MAXM)
	bad("Too many edges");
    }
  if ((long long)N*(Tmax+dmax) > 1000000000)
    bad("Botched invariant #23");
}

static void hinsert(int v)
{
  h[++H] = v;
  int i = H;
  while (i > 1 && dist[h[i]] < dist[h[i/2]])
    {
      int j = h[i/2];
      h[i/2] = h[i];
      h[i] = j;
      i /= 2;
    }
}

static int hdelmin(void)
{
  int z = h[1];
  h[1] = h[H--];
  int i = 1;
  for (;;)
    {
      int j = 2*i;
      if (j > H)
	break;
      if (j+1 <= H && dist[h[j+1]] < dist[h[j]])
	j++;
      if (dist[h[i]] <= dist[h[j]])
	break;
      int k = h[i];
      h[i] = h[j];
      h[j] = k;
      i = j;
    }
  return z;
}

static int findpath(int T)
{
	int i, j;
  H = 0;
  for (i=1; i<=N; i++)
    {
      dist[i] = INF;
      done[i] = 0;
    }
  dist[X] = 0;
  hinsert(X);
  while (H)
    {
      int i = hdelmin();
      //printf("at %d (%d) done=%d H=%d\n", i, dist[i], done[i], H);
      if (done[i])
	continue;
      done[i] = 1;
      for (j=first[i]; j; j=edges[j].next)
	{
	  int d = dist[i] + edges[j].len + (edges[j].delta ? T : 0);
	  int k = edges[j].to;
	  //printf("-> %d, d=%d\n", j, d);
	  if (dist[k] > d)
	    {
	      dist[k] = d;
	      from[k] = i;
	      hinsert(k);
	    }
	}
    }
  return dist[Y];
}

static int getnum(FILE *f)
{
  static int numcount;
  if (numcount++ > 1000000)
    bad("Botched invariant #23");
  for(;;)
    {
      int c = fgetc(f);
      if (c < 0)
	return -3;
      if (c == '\r')
	continue;
      if (c == '\n')
	return -2;
      if (c != ' ' && c != '\t')
	{
	  ungetc(c, f);
	  break;
	}
    }
  int n;
  if (fscanf(f, "%d", &n) != 1)
    bad("Malformed number");
  if (n >= -1)
    return n;
  bad("Unexpected negative number");
}

static int readtime(FILE *f)
{
  int t = getnum(f);
  if (t < -1)
    return -1;
  if (t < 0)
    bad("Reporting negative time");
  if (t > Tmax)
    bad("Reporting time %d>Tmax", t);
  return t;
}

static void readpath(FILE *f, int *plen, int *pdelta)
{
  int i, pcnt = 0;
  for(;;)
    {
      int x = getnum(f);
      if (x == -2)
	break;
      if (x < 1 || x > N)
	bad("Invalid vertex %d", x);
      if (pcnt >= N)
	bad("Path contains too many vertices");
      path[pcnt++] = x;
    }
  if (pcnt < 2 || path[0] != X || path[pcnt-1] != Y)
    bad("Path doesn't lead between X and Y");
  path[pcnt] = 0;

  *plen = 0;
  *pdelta = 0;
  for (i=1; i<=N; i++)
    h[i] = 0;
  h[path[0]] = 1;
  for (i=1; i<pcnt; i++)
    {
      if (h[path[i]])
	bad("Not a path: %d repeated", path[i]);
      h[path[i]] = 1;
      int j = first[path[i-1]];
      while (j && edges[j].to != path[i])
	j = edges[j].next;
      if (!j)
	bad("(%d,%d) is not an edge", path[i-1], path[i]);
      *plen += edges[j].len;
      *pdelta += edges[j].delta;
    }
}

static void checkpath(int T, int i, int pl, int pd, int ok)
{
  int d = findpath(T);
  int his = pl + T*pd;
  //printf("Path #%d at T=%d: given %d, optimal %d\n", i, T, his, d);
  if ((d == his) != ok)
    bad("Inconsistency for path #%d at T=%d: given %d, optimal %d", i, T, his, d);
}

static void check(FILE *f)
{
  int d = findpath(0);
  int c = fgetc(f);
  if (c == 'E')
    {
      if (d < INF)
	bad("Claims no solution, but there is one");
      return;
    }
  ungetc(c, f);
  if (d >= INF)
    bad("Claims solution, but none exists");

  int pl, pd, pi=0;
  int tprev = readtime(f);
  if (tprev < 0)
    bad("Empty list of paths, although solution claimed");
  if (tprev)
    bad("T0 is not 0");
  readpath(f, &pl, &pd);
  if (pl != d)
    bad("First path is not optimal at T=0");

  for(;;)
    {
      int tnext = readtime(f);
      if (tnext < 0)
	break;
      if (tnext <= tprev)
	bad("Times are not increasing");
      checkpath(tnext-1, pi, pl, pd, 1);
      checkpath(tnext, pi, pl, pd, 0);
      readpath(f, &pl, &pd);
      pi++;
      checkpath(tnext, pi, pl, pd, 1);
      tprev = tnext;
    }
  checkpath(Tmax-1, pi, pl, pd, 1);
}

int main(int argc, char **argv)
{
  char *in = argv[1];
  char *out = argv[2];

  FILE *f = fopen(in, "r");
  if (!f)
    bad("Cannot open input file");
  readin(f);
  fclose(f);

  f = fopen(out, "r");
  if (!f)
    bad("Cannot open output file");
  check(f);
  fclose(f);

  puts("OK");
  return 0;
}
