
#include "heuristic.h"

#include <stdlib.h>
#include <string.h>

typedef struct heu_tag {
	int score;
	int cnt;
	int index;
	node_t *node;
	char need[ 4 ];
	char danger;
} heu_tag;

static heu_tag **heap;
static int heap_end;

static void cntNode( heu_tag *tag ) {
	tag->score = 5;
	tag->cnt = 0;
	for( int i = 0; i < tag->node->ncount; ++ i ) {
		node_t *nei = tree.nodes + tag->node->neighbors[ i ];
		heu_tag *ntag = nei->tag;
		int found = 0;
		for( int j = 0; j < 4; ++ j )
			if( tag->node->avail[ j ] && ntag->need[ j ] ) {
				found = 1;
				break;
			}
		if( !found )
			continue;
		if( ntag->danger < tag->score ) {
			tag->score = ntag->danger;
			tag->cnt = 1;
		} else if( ntag->danger == tag->score ) {
			tag->cnt ++;
		}
	}
}

static int tagCmp( heu_tag *_1, heu_tag *_2 ) {
	if( _1->score < _2->score )
		return -2;
	else if( _1->score > _2->score )
		return 2;
	else if( _1->cnt > _2->cnt )
		return -1;
	else if( _1->cnt < _2->cnt )
		return 1;
	else if( _1->danger > _2->danger )
		return -1;
	else if( _1->danger < _2->danger )
		return 1;
	else
		return 0;
}

static void swap( heu_tag **_1, heu_tag **_2 ) {
	heu_tag *tmp = *_2;
	*_2 = *_1;
	*_1 = tmp;
	int itmp = ( *_2 )->index;
	( *_2 )->index = ( *_1 )->index;
	( *_1 )->index = itmp;
}

static void updateHeap( int index ) {
	if( index >= heap_end )
		return;
	int lower;
	if( index && ( tagCmp( heap[ index ], heap[ lower = ( index - 1 ) / 2 ] ) < 0 ) ) {
		while( index && ( tagCmp( heap[ index ], heap[ lower = ( index - 1 ) / 2 ] ) < 0 ) ) {
			swap( heap + index, heap + lower );
			index = lower;
		}
	} else {
		int cont = 1;
		while( cont ) {
			int left = 2 * index + 1,
				right = 2 * index + 2;
			if( left >= heap_end )
				left = 0;
			if( right >= heap_end )
				right = 0;
			cont = ( ( left && ( tagCmp( heap[ index ], heap[ left ] ) > 0 ) ) ||
					( right && ( tagCmp( heap[ index ], heap[ right ] ) > 0 ) ) );
			int with = ( left && right && ( tagCmp( heap[ left ], heap[ right ] ) > 0 ) ) ? right : left;
			if( cont ) {
				swap( heap + index, heap + with );
				index = with;
			}
		}
	}
}

void heu_init( void ) {
	heu_tag *buffer = malloc( tree.N * sizeof *buffer );
	heap = malloc( tree.N * sizeof *heap );
	heap_end = tree.N;
	for( int i = 0; i < tree.N; ++ i ) {
		tree.nodes[ i ].tag = buffer;
		buffer->danger = ( tree.nodes[ i ].ncount >= 4 ) ? 4 : 5;
		buffer->node = tree.nodes + i;
		buffer->index = i;
		memset( buffer->need, tree.nodes[ i ].ncount >= 4, sizeof buffer->need );
		buffer ++;
	}
	for( int i = 0; i < tree.N; ++ i ) {
		cntNode( tree.nodes[ i ].tag );
	}
	for( int i = 0; i < tree.N; ++ i ) {
		heap[ i ] = tree.nodes[ i ].tag;
		heap_end = i + 1;
		updateHeap( i );
	}
}

static void decorate( move_t move ) {
	heu_tag *tag = move.node->tag;
	tag->danger = 5;
	tag->score = 5;
	tag->cnt = 0;
	memset( tag->need, 0, sizeof tag->need );
	updateHeap( tag->index );
	for( int i = 0; i < move.node->ncount; ++ i ) {
		node_t *nei = tree.nodes + move.node->neighbors[ i ];
		heu_tag *ntag = nei->tag;
		cntNode( ntag );
		updateHeap( ntag->index );
		if( ntag->need[ ( unsigned ) move.deco ] ) {
			if( ! ( -- ntag->danger ) )
				ntag->danger = 5;
			ntag->need[ ( unsigned ) move.deco ] = 0;
			for( int j = 0; j < nei->ncount; ++ j ) {
				node_t *snei = tree.nodes + nei->neighbors[ j ];
				heu_tag *stag = snei->tag;
				cntNode( stag );
				updateHeap( stag->index );
			}
		}
	}
}

move_t heu_move( move_t move ) {
	decorate( move );
	for(;;) {
		heu_tag *tag = *heap;
		swap( heap, heap + ( -- heap_end ) );
		updateHeap( 0 );
		if( tag->node->deco != 5 )
			continue;
		int desco[ 4 ];
		memset( desco, 0, sizeof desco );
		for( int i = 0; i < tag->node->ncount; ++ i ) {
			for( int j = 0; j < 4; ++ j ) {
				heu_tag *nei = tree.nodes[ tag->node->neighbors[ i ] ].tag;
				if( tag->node->avail[ j ] && nei->need[ j ] )
					desco[ j ] ++;
			}
		}
		int best = 0;
		for( int i = 1; i < 4; ++ i )
			if( ( desco[ i ] > desco[ best ] ) || ( ( ! desco[ best ] ) && tag->node->avail[ i ] ) )
				best = i;
		move.node = tag->node;
		move.deco = best;
		decorate( move );
		return move;
	}
}
