unit tree_lib_demo;

interface

	function init: longint;

	function neighbor_count( point: longint ): longint;
	function neighbor( point, index: longint ): longint;

	function decoration( point: longint ): char;

	function enemy: longint;
	procedure decorate( point: longint; decoration: char );

implementation

type
	point_t = record
		neighbor_count: longint;
		neighbors: array[ 1..10 ] of longint;
		decoration: char;
	end;

var
	N: longint = 0;
	rest: longint;
	points: array[ 1..10 ] of point_t;
	move: boolean;
	types: array[ 1..4 ] of char = ( 'A', 'B', 'D', 'S' );

procedure die( msg: string );
begin
	writeLn( msg );
	halt( 1 );
end;

function init: longint;
var
	i: longint;
begin
	if N > 0 then die( 'Init already called' );
	N := 8;
	points[ 1 ].neighbor_count := 1;
	points[ 1 ].neighbors[ 1 ] := 2;
	points[ 2 ].neighbor_count := 4;
	points[ 2 ].neighbors[ 1 ] := 1;
	points[ 2 ].neighbors[ 2 ] := 3;
	points[ 2 ].neighbors[ 3 ] := 4;
	points[ 2 ].neighbors[ 4 ] := 5;
	points[ 3 ].neighbor_count := 1;
	points[ 3 ].neighbors[ 1 ] := 2;
	points[ 4 ].neighbor_count := 1;
	points[ 4 ].neighbors[ 1 ] := 2;
	points[ 5 ].neighbor_count := 4;
	points[ 5 ].neighbors[ 1 ] := 2;
	points[ 5 ].neighbors[ 2 ] := 6;
	points[ 5 ].neighbors[ 3 ] := 7;
	points[ 5 ].neighbors[ 4 ] := 8;
	points[ 6 ].neighbor_count := 1;
	points[ 6 ].neighbors[ 1 ] := 5;
	points[ 7 ].neighbor_count := 1;
	points[ 7 ].neighbors[ 1 ] := 5;
	points[ 8 ].neighbor_count := 1;
	points[ 8 ].neighbors[ 1 ] := 5;
	for i := 1 to N do
		points[ i ].decoration := ' ';
	move := true;
	rest := N;
	init := N;
end;

function neighbor_count( point: longint ): longint;
begin
	if N = 0 then die( 'No init called' );
	if ( point < 1 ) or ( point > N ) then
		neighbor_count := 0
	else
		neighbor_count := points[ point ].neighbor_count;
end;

function neighbor( point, index: longint ): longint;
begin
	if N = 0 then die( 'No init called' );
	if ( point < 1 ) or ( point > N ) then
		neighbor := 0
	else if ( index < 1 ) or ( index > points[ point ].neighbor_count ) then
		neighbor := 0
	else
		neighbor := points[ point ].neighbors[ index ];
end;

function decoration( point: longint ): char;
begin
	if N = 0 then die( 'No init called' );
	if ( point < 1 ) or ( point > N ) then
		decoration := ' '
	else
		decoration := points[ point ].decoration;
end;

function enemy: longint;
var
	i, j, k, neigh: longint;
	avail: array[ 1..4 ] of boolean;
begin
	if N = 0 then die( 'No init called' );
	if move then die( 'It is your turn' );
	move := not move;
	for i := 1 to N do begin
		for j := 1 to 4 do avail[ j ] := true;
		if points[ i ].decoration <> ' ' then
			continue;
		for j := 1 to points[ i ].neighbor_count do begin
			neigh := points[ i ].neighbors[ j ];
			if points[ neigh ].decoration = ' ' then
				continue;
			for k := 1 to 4 do
				if points[ neigh ].decoration = types[ k ] then begin
					avail[ k ] := false;
					break;
				end;
		end;
		for j := 1 to 4 do
			if avail[ j ] then begin
				points[ i ].decoration := types[ j ];
				writeLn( i, ' ', types[ j ] );
				dec( rest );
				if rest = 0 then
					halt( 0 );
				enemy := i;
				exit;
			end;
	end;
	die( 'Enemy could not move' );
end;

procedure decorate( point: longint; decoration: char );
var
	found: boolean;
	i: longint;
begin
	if N = 0 then die( 'No init called' );
	if not move then die( 'Not your move' );
	move := not move;
	if ( point < 1 ) or ( point > N ) then die( 'No such point' );
	found := false;
	for i := 1 to 4 do
		if types[ i ] = decoration then begin
			found := true;
			break;
		end;
	if not found then die( 'Unknown decoration type' );
	for i := 1 to points[ point ].neighbor_count do
		if decoration = points[ points[ point ].neighbors[ i ] ].decoration then
			die( 'Ilegal decoration' );
	points[ point ].decoration := decoration;
	writeLn( point, ' ', decoration );
	dec( rest );
	if rest = 0 then
		halt( 0 );
end;

end.
