/*

************************************
**   OIBH May Day Contest 2003    **
**                                **
**    C: K-th Comfortable Route   **
**        Solution Program        **
**                                **
**      Author: Comars Zhao       **
**                                **
**         May 1st, 2003          **
************************************

*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <algorithm>

struct Tedge {
	int x, y;
	int v;

	friend const bool operator< (Tedge a, Tedge b) {
		return a.v<b.v;
	}
};

struct Tnum {
	int x, y;

	friend const bool operator== (Tnum a, Tnum b) {
		__int64 v1 = (__int64)a.x*b.y, v2 = (__int64)b.x*a.y;
		return v1==v2;
	}

	friend const bool operator< (Tnum a, Tnum b) {
		__int64 v1 = (__int64)a.x*b.y, v2 = (__int64)b.x*a.y;
		return v1<v2;
	}
};

FILE *fin  = fopen("C.DAT", "r+");
FILE *fout = fopen("C.OUT", "w+");

const Tnum INFINITE = { 2147483647, 1 };

void init();
void solve();
int getroot(int r);
void update(Tnum v);
int gcd(int m, int n);

int n, m;
Tedge edge[5000];
int s, t, K;

int prev[501];

Tnum res[50];
int ntmp;
Tnum tmp[500];

int main() {
	init();
	solve();

	fclose(fin);
	fclose(fout);

	return 0;
}

void init() {
	fscanf(fin, "%d %d", &n, &m);
	for ( int i=0; i<m; ++i ) {
		int x, y, v;

		fscanf(fin, "%d %d %d", &x, &y, &v);
		edge[i].x=x; edge[i].y=y; edge[i].v=v;
	}
	fscanf(fin, "%d %d %d", &s, &t, &K);

	std::sort(edge, edge+m);
}

void solve() {
	int p, q;

	for ( int i=0; i<K; ++i )
		res[i]=INFINITE;

	for ( p=0; p<m; ++p ) {
		memset(prev, 0, (n+1)*sizeof(int));

		q=p;
		while ( q<m && getroot(s)!=getroot(t) ) {
			int r1 = getroot(edge[q].x), r2 = getroot(edge[q].y);
			if ( r1!=r2 )
				prev[r2]=r1;

			++q;
		}

		if ( getroot(s)!=getroot(t) )
			break;

		if ( q>p ) --q;
		while ( q<m ) {
			int r1 = getroot(edge[q].x), r2 = getroot(edge[q].y);
			if ( r1!=r2 )
				prev[r2]=r1;

			Tnum v = {edge[q].v, edge[p].v};
			if ( res[K-1]<v )
				break;

			if ( getroot(s)==getroot(edge[p].x) && getroot(s)==getroot(edge[q].x) && getroot(s)==getroot(t) )
				update(v);

			++q;
		}
	}

	if ( res[K-1]<INFINITE ) {
		int d = gcd(res[K-1].x, res[K-1].y);
		res[K-1].x/=d; res[K-1].y/=d;

		if ( res[K-1].y==1 )
			fprintf(fout, "%d\n", res[K-1].x);
		else
			fprintf(fout, "%d/%d\n", res[K-1].x, res[K-1].y);
	}
	else
		fprintf(fout, "IMPOSSIBLE\n");
}

int getroot(int r) {
	int x = r;
	while ( prev[x] )
		x=prev[x];

	while ( r!=x ) {
		int p = prev[r]; prev[r]=x;
		r=p;
	}

	return x;
}

void update(Tnum v) {
	for ( int i=0; i<K; ++i )
		if ( v<res[i] ) {
			memmove(res+i+1, res+i, (K-i-1)*sizeof(Tnum));
			res[i]=v;
			break;
		}
		else if ( res[i]==v )
			break;
}

int gcd(int m, int n) {
	if ( n==0 )
		return m;
	return gcd(n, m%n);
}