/* Sea2 O(NlogN) AVL Solution by Berinde Radu */

#include <stdio.h>
#include <stdlib.h>

typedef struct node
{
    int x, y;
    struct node *l, *r;
    int height, nr;
} node;
#define max(a, b) ((a) > (b) ? (a) : (b))
#define getheight(n) (n->height = max(n->l->height, n->r->height) + 1, \
                      n->nr = n->l->nr + n->r->nr + 1)

node *Root, *NIL;

node *rotl(node *k1)
{
    node *k2 = k1->l;
    k1->l = k2->r;
    k2->r = k1;
    getheight(k1);
    getheight(k2);
    return k2;
}

node *rotr(node *k2)
{
    node *k1 = k2->r;
    k2->r = k1->l;
    k1->l = k2;
    getheight(k2);
    getheight(k1);
    return k1;
}

node *balance(node *n)
{
    if (n == NIL) return NIL;
    getheight(n);
    if (n->l->height > n->r->height + 1)
    {
        if (n->l->r->height > n->l->l->height)
           n->l = rotr(n->l);
        n = rotl(n);
    }
    if (n->r->height > n->l->height + 1)
    {
        if (n->r->l->height > n->r->r->height)
           n->r = rotl(n->r);
        n = rotr(n);
    }
    return n;
}

node *insert(node *n, int x, int y)
{
    if (n == NIL)
    {
        n = (node *) calloc(1, sizeof(node));
        n->height = n->nr = 1, n->x = x, n->y = y;
        n->l = n->r = NIL;
        return n;
    }
    if (x < n->x)
        n->l = insert(n->l, x, y);
    else
        n->r = insert(n->r, x, y);
    return balance(n);
}

node *erase(node *n, int x)
{
    node *t;
    if (n->x == x)
    {
        if (n->l == NIL || n->r == NIL)
        {
            t = n->l == NIL ? n->r : n->l;
            free(n);
            return t;
        }
        for (t = n->r; t->l != NIL; t = t->l);
        n->x = t->x, n->y = t->y;
        n->r = erase(n->r, t->x);
        return balance(n);
    }
    if (x < n->x)
        n->l = erase(n->l, x);
    else
        n->r = erase(n->r, x);
    return balance(n);
}

node *findbefore(node *n, int x)
{
    node *r;
    if (n == NIL) return NIL;
    if (n->x > x)
        return findbefore(n->l, x);
    r = findbefore(n->r, x);
    return r == NIL ? n : r;
}

node *findafter(node *n, int x)
{
    node *r;
    if (n == NIL) return NIL;
    if (n->x < x)
        return findafter(n->r, x);
    r = findafter(n->l, x);
    return r == NIL ? n : r;
}

void init()
{
    Root = NIL = (node *) calloc(1, sizeof(node));
}

void read_and_solve()
{
    FILE *fi = fopen("sea2.in", "rt");
    FILE *fo = fopen("sea2.out", "wt");
    int x, y, nr;
    node *n;
    fscanf(fi, "%d", &nr);
    init();
    while (nr--)
    {
        fscanf(fi, "%d %d", &x, &y);
	if (x < 1 || x > 260000 || y < 1 || y > 260000) fprintf(fo, "wrong test\n");
        n = findafter(Root, x);
        if (n != NIL && n->y > y)
        {
            fprintf(fo, "-1\n");
            continue;
        }
        while ((n = findbefore(Root, x)) != NIL && n->y < y)
            Root = erase(Root, n->x);
        Root = insert(Root, x, y);
        fprintf(fo, "%d\n", Root->nr);
    }
    fclose(fo);
    fclose(fi);
}

int main()
{
    read_and_solve();
    return 0;
}
